#!/usr/bin/python3
# -*- coding: UTF-8 -*-
#
#       copyFilesFromTempToCurrDir.py
#
#           tempFolder内のfileをcurrDirに転送する。
#           　cntファイル：等分布荷重のセットがあれば、それをセット
#           　mshファイル：data省略形から内容を展開してセット
#
#
#   19/03/07    新規作成
#      03/14    isLatestFile: otherFilesをコピーする時、最新fileのみコピーする様に修正
#               （currDirの内容が旧のfileに置き換わってしまう為。）
#               copyFilesFromTempToCurrDir: timeStamp（tempCaseDate）の設定方法を修正
#               （保存しても、timeStampが変わらなかった為）
#

import sys
import os
import glob
import shutil
import time

import pyFistr

ls = "\n"
binAppPath = pyFistr.binAppPath
pythonAppPath = pyFistr.pythonAppPath

#tempFolderにコピーする時、shrinkするkeywordを取得（これ以外はそのままコピーする）
shrinkKeywords = pyFistr.shrinkKeywords


#
#  copyCntFromTempToCurrDir
#    temp内のcntファイルに等分布荷重を追加しcurrDirに保存する
def copyCntFromTempToCurrDir():
    tempFile = currDirTemp + os.sep + "FistrModel.cnt"
    cntFile = "FistrModel.cnt"
    #  等分布荷重を設定して保存
    comm = "python3 " + binAppPath + "addTrueForceInCntFile.py " + tempFile + " " + cntFile
    os.system(comm)

#
#  copyMshFromTempToCurrDir
#    temp内のmshファイルから実mshを作成しcurrDirに保存する
def copyMshFromTempToCurrDir():
    #FistrModel.mshを転送
    meshHeaderConts = createHeaderDataFromTemp()
    lines = []
    for i in range(len(meshHeaderConts)):
        headerCont = meshHeaderConts[i]
        header = headerCont[0]
        lines += [header]
        if len(headerCont[1]) != 0:
            lines += headerCont[1]
    fileName = currDir + os.sep + "FistrModel.msh"
    f = open(fileName, "w", encoding="utf-8")
    for line in lines:
        f.write(line)
    f.close()
    

#
#  createHeaderDataFromTemp
#    tempから実meshHeaderDataを作成する
def createHeaderDataFromTemp():

    def getNextHeader(meshHeaderConts, keyword, j):
        cont = []
        while j < len(meshHeaderConts):
            header = meshHeaderConts[j][0]
            words = pyFistr.deleteSp(header).split(",")
            if words[0] == keyword:
                cont = meshHeaderConts[j][1]
                break
            j += 1
        return [cont, j]

    tempFile = currDirTemp + os.sep + "FistrModel.msh"
    f=open(tempFile, encoding="utf-8"); lines = f.readlines(); f.close()
    meshHeaderConts = pyFistr.getHeaderData(lines)
    trueFile = currDir + os.sep + "FistrModel.msh"
    f = open(trueFile, encoding="utf-8"); lines = f.readlines(); f.close()
    trueMeshHeaderConts = pyFistr.getHeaderData(lines)
    #入れ替えるcardを定義（定義されているcardは、実folderの内容を使用する）
    #changeCards = ["!NODE", "!ELEMENT", "!EGROUP", "!SGROUP", "!NGROUP", "!EQUATION"]
    changeCards = shrinkKeywords
    #入れ替え
    j = 0
    for i in range(len(meshHeaderConts)):
        headerCont = meshHeaderConts[i]
        header = headerCont[0]
        words = pyFistr.deleteSp(header).split(",")
        for changeCard in changeCards:
            if words[0].upper() == changeCard:
                [cont, j] = getNextHeader(trueMeshHeaderConts, changeCard, j)
                j += 1
                meshHeaderConts[i][1] = cont
                break
    return meshHeaderConts

#
#  getMeshContsFromTempFolder
#    tempFolderからその内容を取得する
def getMeshContsFromTempFolder():
    meshConts = {}
    meshConts["currDir"] = os.getenv("HOME")
    fileName = currDirTemp + os.sep + "meshConts"
    f = open(fileName, encoding="utf-8")
    lines = f.readlines()
    f.close()
    flag = ""
    for i in range(len(lines)):
        line = lines[i]
        if line[:len("*currDir")] == "*currDir":
            flag = "currDir"
            meshConts["currDir"] = ""
        elif line[:len("*meshCont")] == "*meshCont":
            flag = "meshCont"
            meshConts["meshCont"] = ""
        elif line[:len("*ampNames")] == "*ampNames":
            flag = "ampNames"
            meshConts["ampNames"] = []
        elif line[:len("*initialNames")] == "*initialNames":
            flag = "initialNames"
            meshConts["initialNames"] = []
        elif line[:len("*overlapNGRP")] == "*overlapNGRP":
            flag = "overlapNGRP"
            meshConts["overlapNGRP"] = []
        elif line[:len("*tempCaseDate")] == "*tempCaseDate":
            flag = "tempCaseDate"
            meshConts["tempCaseDate"] = ""
        else:
            if flag == "currDir" and len(line.split()) > 0:
                meshConts["currDir"] = line.split()[0]
            elif flag == "meshCont":
                meshConts["meshCont"] += line
            elif flag == "ampNames" and len(line.split()) > 0:
                meshConts["ampNames"] += [line.split()[0]]
            elif flag == "initialNames" and len(line.split()) > 0:
                meshConts["initialNames"] += [line.split()[0]]
            elif flag == "overlapNGRP" and len(line.split()) > 0:
                meshConts["overlapNGRP"] += [line.split()[0]]
            elif flag == "tempCaseDate" and len(line.split()) > 0:
                meshConts["tempCaseDate"] = float(line.split()[0])
    return meshConts

#  saveCaseContDict
#    dict形式のdataを保存する
def saveCaseContDict(dataDict):
    cont = ""
    items = list(dataDict.keys())
    items.sort()
    for item in items:
        value = dataDict[item]
        if type(value) == list:
            value = "\n".join(value)
        elif type(value) == float or type(value) == int:
            value = str(value)
        else:
            if value[-1] == "\n":
                value = value[:-1]
        cont += "*" + item + "\n" + value + "\n"
    #fileに書き込み
    fileName = currDirTemp + os.sep + "meshConts"
    f = open(fileName, "w", encoding="utf-8")
    f.write(cont)
    f.close()

#
#  isLatestFile
#    最新fileかどうかチェック
def isLatestFile(fileName, currDir):
    name = fileName.split(os.sep)[-1]
    fileDate = os.path.getmtime(fileName)
    currFile = currDir + os.sep + name
    if len(glob.glob(currFile)) == 0:
        return True
    currDate = os.path.getmtime(currFile)
    return fileDate > currDate

#
#  remakeTimeStampVtuMeshFile
def remakeTimeStampVtuMeshFile(currDir):
    """ vtuのmeshFileのtimeStampを修正する。"""
    vtuFile = currDir + os.sep + "convFistrModel.msh.vtu"
    if os.path.exists(vtuFile):
        atime = mtime = time.time()
        os.utime(vtuFile, (atime, mtime))
        print("changed time stamp of vtuMeshFile.")

#
#  copyFilesFromTempToCurrDir
#  --------------------------
#    tempFolder内のfileをcurrDirに転送する
def copyFilesFromTempToCurrDir(currDirTemp, currDir):
    #その他fileを転送
    copyFiles = []
    files = glob.glob(currDirTemp + os.sep + "*")
    for fileName in files:
        if os.path.isfile(fileName) == True:
            name = fileName.split(os.sep)[-1]
            if (name == "FistrModel.cnt"
                or name == "FistrModel.msh"
                or name == "meshConts"
                or name.split(".")[-1] == "bat"
                or name == "run"):
                pass
            else:
                if isLatestFile(fileName, currDir) == True:
                    copyFiles.append(fileName)
    #  転送前に「hecmw_part_ctrl.dat」を削除
    fileName = currDir + os.sep + "hecmw_part_ctrl.dat"
    if len(glob.glob(fileName)) != 0: 
        os.remove(currDir + os.sep + "hecmw_part_ctrl.dat")
    #  fileを転送
    for fileName in copyFiles:
        shutil.copy(fileName, currDir)
    #FistrModel.cntを転送（等分布荷重をセット）
    print (ls+"Copying cnt file...")
    copyCntFromTempToCurrDir()
    #FistrModel.mshを転送（実メッシュを作成）
    print (ls+"Copying msh file...")
    copyMshFromTempToCurrDir()
    #tempCaseDateを更新（timeStampを更新）
    meshConts = getMeshContsFromTempFolder()
    #tempCaseDate = os.path.getmtime(currDirTemp + os.sep + "FistrModel.cnt") + 1.0
    tempCaseDate = time.time() + 1.0
    meshConts["tempCaseDate"] = tempCaseDate
    saveCaseContDict(meshConts)
    #vtu形式のmeshFileのtimeStampを修正
    remakeTimeStampVtuMeshFile(currDir)
    print (ls+"Copied cnt, msh and other files to currDir.")
    print ("Finished to prepare for FrontISTR.", ls)
    

if __name__ == "__main__":
    currDirTemp = sys.argv[1]
    currDir = sys.argv[2]
    copyFilesFromTempToCurrDir(currDirTemp, currDir)

