#!/usr/bin/python3  
# -*- coding: utf-8 -*-    
#
#       deleteFilesDialog.py
#
#           fileを削除する
#
#   15/06/16    新規作成
#      09/01    windows対応のため、os.sepに修正
#      12/18    windows版のpangoWarning対策で、初期化を修正
#      12/25    行末コードlsを修正
#   19/01/09    python3用に書き換え
#      01/26    okCancelDialog:type→message_typeに修正
#   20/02/28    FrontISTR-5対応で修正
#      04/02    削除せず残すfileを追加。
#
#

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk

import sys   
import os   
import os.path   
import glob
import shutil

builder = ""
mainWindow = ""

ls = "\n"

#  remainFiles
#    削除せずに残すfile名
def remainFiles():
    files = ["convFistrModel.msh.vtu"]
    return files

#  partMeshFiles
#    並列処理用に分割したメッシュファイル
def partMeshFiles():
    files = ["FistrModel_p*"]
    return files

#  currResultFolders
#    現在の結果フォルダ
def currResultFolders():
    folders = ["FistrModel.*"]
    return folders

#  currResultFiles
#    現在の結果ファイル
def currResultFiles():
    files = ["FistrModel.res.*", "FistrModel*.inp", "FistrModel.restart.*", "*.vtk",
             "conv*", "*.log", "FSTR.dbg.*", "FSTR.msg", "FSTR.sta",
             "*.pvtu", "*.vtu"]
    return files

#  eigenResultFiles
#    固有値解析結果ファイル
def eigenResultFiles():
    files = ["FistrModel_eigen.res.*", "eigen_log"]
    return files

#  responseResultFiles
#    周波数応答解析結果ファイル
def responseResultFiles():
    files = ["FistrModel_dyna.res.*", "dyna_*.txt"]
    return files

#  heatResultFiles
#    熱伝導解析結果ファイル
def heatResultFiles():
    files = ["FistrModel_temp.res.*"]
    return files


#  ---------
#  windowApp
#  ---------
#    GUIの起動と終了
class windowApp:

    #  GUIの初期化
    def __init__(self):
        global builder, mainWindow
        builder = Gtk.Builder()
        #builder.add_from_file(os.getenv("easyIstrPath") + os.sep + "easyIstr_2.glade")
        builder.add_from_file(os.path.dirname(os.path.abspath(__file__)) + os.sep + "deleteFilesDialog2.glade")
        mainWindow = builder.get_object("window1")
        mainWindow.connect("delete-event", self.close)
        builder.connect_signals(self)

        self.setGtkObject()
        self.subInitialize()
        self.setTitle()


    #  GUIの表示
    def main(self):
        mainWindow.show_all()
        Gtk.main()

    #
    #  GUIの終了
    #
    def close(self, *args):
        Gtk.main_quit(*args)

    #
    #  handler
    #  ------------------------------
    #削除
    def onDeleteFiles(self, event):
        self.deleteFiles(event)
        Gtk.main_quit()
    #キャンセル
    def onCancel(self, event):
        Gtk.main_quit()
    #--------------------------------

    #
    #  setGtkObject
    #    objectの定義
    def setGtkObject(self):
        self.checkBtn_partMesh = builder.get_object("checkbutton1")
        self.checkBtn_currRes = builder.get_object("checkbutton2")
        self.checkBtn_eigenRes = builder.get_object("checkbutton3")
        self.checkBtn_respRes = builder.get_object("checkbutton4")
        self.checkBtn_heatRes = builder.get_object("checkbutton5")
        self.label_message = builder.get_object("label1")
        self.label_partMesh = builder.get_object("label2")
        self.label_currRes = builder.get_object("label3")
        self.label_eigenRes = builder.get_object("label4")
        self.label_respRes = builder.get_object("label5")
        self.label_heatRes = builder.get_object("label6")

    #
    #  subInitialize
    #    初期化
    def subInitialize(self):
        #labelを設定
        label = _(u"ファイルを削除します")
        label += ls + _(u"削除するファイルを選択してください")
        self.label_message.set_text(label)
        label = makeLabel(partMeshFiles())
        self.label_partMesh.set_text(label)
        label = makeLabel(currResultFiles())
        self.label_currRes.set_text(label)
        label = makeLabel(eigenResultFiles())
        self.label_eigenRes.set_text(label)
        label = makeLabel(responseResultFiles())
        self.label_respRes.set_text(label)
        label = makeLabel(heatResultFiles())
        self.label_heatRes.set_text(label)
        #checkBtnの設定
        self.checkBtn_partMesh.set_sensitive(False)
        files = partMeshFiles()
        for name in files:
            if len(glob.glob(name)) != 0:
                self.checkBtn_partMesh.set_sensitive(True)
                break
        self.checkBtn_currRes.set_sensitive(False)
        files = currResultFiles()
        for name in files:
            if len(glob.glob(name)) != 0:
                self.checkBtn_currRes.set_sensitive(True)
                self.checkBtn_currRes.set_active(True)
                break
        self.checkBtn_eigenRes.set_sensitive(False)
        files = eigenResultFiles()
        for name in files:
            if len(glob.glob(name)) != 0:
                self.checkBtn_eigenRes.set_sensitive(True)
                break
        self.checkBtn_respRes.set_sensitive(False)
        files = responseResultFiles()
        for name in files:
            if len(glob.glob(name)) != 0:
                self.checkBtn_respRes.set_sensitive(True)
                break
        self.checkBtn_heatRes.set_sensitive(False)
        files = heatResultFiles()
        for name in files:
            if len(glob.glob(name)) != 0:
                self.checkBtn_heatRes.set_sensitive(True)
                break

    #
    #  setTitle
    #    GUIのタイトルを設定
    def setTitle(self):
        title = _(u"不要なファイルを削除")
        mainWindow.set_title(title)

    #
    #  deleteFiles
    #    指定されたfileを削除
    def deleteFiles(self, event):
        #確認メッセージ
        title = _(u"ファイルの削除")
        mess = _(u"本当に削除しますか？") + ls
        mess += _(u"削除すると元に戻りません")
        stat = self.okCancelDialog(title, mess)
        if stat != "OK":
            return

        #ファイルの削除
        if self.checkBtn_partMesh.get_active() == True:
            deleteSetNames(partMeshFiles())
        if self.checkBtn_currRes.get_active() == True:
            deleteSetFolders(currResultFolders())
            deleteSetNames(currResultFiles())
        if self.checkBtn_eigenRes.get_active() == True:
            deleteSetNames(eigenResultFiles())
        if self.checkBtn_respRes.get_active() == True:
            deleteSetNames(responseResultFiles())
        if self.checkBtn_heatRes.get_active() == True:
            deleteSetNames(heatResultFiles())
        print (_(u"ファイルを削除しました"))


    def okCancelDialog(self, title, message):
        dlg = Gtk.MessageDialog(
                parent=mainWindow,
                message_type=Gtk.MessageType.INFO,
                buttons=Gtk.ButtonsType.OK_CANCEL)
        dlg.set_title(title)
        #messageを分ける
        mess1 = message.split(ls)[0]
        mess2 = ls.join(message.split(ls)[1:])
        dlg.set_markup(mess1)
        dlg.format_secondary_text(mess2)
        ans = dlg.run() 
        dlg.destroy()
        if ans == Gtk.ResponseType.OK:
            stat = "OK"
        else:
            stat = "CANCEL"
        return stat

#
#  deleteSetFolders
#    指定されたfolderを削除
def deleteSetFolders(folders):
    for folder in folders:
        delNames = glob.glob(folder)
        for delName in delNames:
            if os.path.isdir(delName) == True:
                shutil.rmtree(delName)

#
#  deleteSetNames
#    指定されたfileを削除
def deleteSetNames(names):
    for name in names:
        comm = "rm -rf " + name
        print (comm)
        delNames = glob.glob(name)
        if len(delNames) > 0:
            for delName in delNames:
                if not delName in remainFiles():
                    os.remove(delName)

#
#  makeLabel
#    単語を接続してlabelを作成
def makeLabel(words):
    label = ""
    maxl = 60
    l = 0
    i = 0
    ip = 0
    while i < len(words):
        l += len(words[i])
        if l > maxl:
            label += ", ".join(words[ip:i])
            if ip < len(words):
                label += ls
            ip = i
            l = 0
        i += 1
    if ip == 0:
        label = ", ".join(words)
    elif ip != i-1:
        label += ", ".join(words[ip:])
    return label




if __name__ == "__main__":
    import gettext
    gettext.install("app")
    _ = gettext.gettext

    app = windowApp()
    app.main()

