#!/usr/bin/python3
#  coding: utf-8
#
#       openFolderDDialog.py
#
#       開くfolderDirを取得する。
#
#   20/03/12    新規作成
#

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, Gdk, GdkPixbuf, Pango

import os
import GtkParts
import pyFistr
import abaqus2fistr

#----------------------
#  dialogApp class
#----------------------
class dialogApp:

    def __init__(self, rootDir, currDir, title, message):
        self.builder = Gtk.Builder()
        path = os.getenv("easyIstrPath") + os.sep + "python" + os.sep
        self.builder.add_from_file(path + "getOpenFolderDDialog.glade")
        self.dialog = self.builder.get_object("dialog1")
        self.dialog.connect("delete-event", self.close)
        self.builder.connect_signals(self)
        self.setGtkObject()
        self.rootDir = rootDir
        self.currDir = currDir
        self.selectDir = ""
        self.title = title
        self.message = message
        self.treeWidget = GtkParts.treeWidget(self.tree_treeDir)
        self.NORMAL = Pango.Weight.NORMAL   #標準
        self.BOLD =   Pango.Weight.BOLD     #bold
        self.dataDict = {}
        #iconの定義
        iconDir = os.getenv("easyIstrPath") + os.sep + "icons"
        iconFile = GdkPixbuf.Pixbuf.new_from_file
        self.iconFolder = iconFile(iconDir + os.sep + "folder16.png")
        self.iconCase = iconFile(iconDir + os.sep + "x-office-document16.png")
        self.iconFolderCk = iconFile(iconDir + os.sep + "folderCheck16.png")
        self.iconCaseCk = iconFile(iconDir + os.sep + "x-office-document_check16.png")
        self.maskEvent = True

    #
    #  show
    #-----------
    def show(self):
        #title, messageを設定
        self.dialog.set_title(self.title)
        self.label_message.set_text(self.message)
        #rootからcurrDirまでのdirを取得
        folders = self.getAllFolders()
        dataDict = self.createDataDict(folders)
        self.treeWidget.dataDict = dataDict
        #treeのitemを定義(item、headerの内容、itemの位置変数を定義)
        self.defineTreeItem()
        #itemの辞書（itemDict）を作成
        itemDict = self.createItemDict(folders)
        self.treeWidget.itemDict = itemDict
        #treeDataを作成
        treeData = self.treeWidget.createTreeData(itemDict)
        self.treeWidget.treeData = treeData
        #treeNodesを作成
        self.treeWidget.createTreeNodes(treeData, self.itemDefines)
        #treeViewを作成
        self.treeWidget.createTreeView(self.headers, self.itemDefines,
                    colWidthes=self.colWidthes, multi=False)
        #currDirまで展開する
        self.treeWidget.expandToCurrDir(self.currDir)
        #currDirを選択する
        self.treeWidget.selectDir(self.currDir)
        #currDirのiconと書体を設定する
        self.setSelectedDir()
        #eventMaskを解除
        self.maskEvent = False
        #dialogを表示
        self.dialog.run()
        return self.selectDir

    #
    #  close
    #-----------
    def close(self, *args):
        self.dialog.destroy()

    #
    #  setGtkObject
    #----------------
    def setGtkObject(self):
        self.tree_treeDir = self.builder.get_object("tree_treeDir")
        self.label_message = self.builder.get_object("label_dialogCont")

    #--------- event handler --------------
    def onApply(self, event):
        self.selectDir = self.treeWidget.getSelectedDir()
        self.close()
    def onCancel(self, event):
        self.selectDir = ""
        self.close()
    def onExpandDir(self, treeview, treeiter, path):
        if self.maskEvent == False:        
            self.expandDir(path)
    #----------------------------------------

    #
    #  getAllFolders
    #----------------
    def getAllFolders(self):
        """ rootからcurrDirまでの全folderDirを取得して返す"""
        folders = self.treeWidget.getDirUnderRootToSelDir(self.rootDir, self.currDir)
        folders += self.treeWidget.getChildFolders(folders)
        folders = list(set(folders))
        folders.sort()
        return folders

    #
    #  createDataDict
    #-----------------
    def createDataDict(self, folders):
        """ treeを作成する為の辞書（dataDict）を作成する"""
        #folderの辞書作成
        dataDict = {}
        for folder in folders:
            folderCont = self.getFolderCont(folder)
            dataDict[folder] = folderCont
        return dataDict

    #
    #  defineTreeItem
    #-----------------
    def defineTreeItem(self):
        """ itemの内容とheaderを定義する。"""
        #itemを定義
        pixbuf = GdkPixbuf.Pixbuf
        weight = Pango.Weight
        #                   icon    dir  solutin     solver    model    size (bold)   color
        self.itemDefines = [pixbuf, str, str,        str,      str,     str,  weight, str]
        self.headers     = ["directory", "solution", "solver", "model", "meshSize(MB)"]
        self.colWidthes  = [200,          None,      None,     100]
        #位置変数を定義
        self.COL_ICON     = 0
        self.COL_DIR      = 1
        self.COL_SOLUTION = 2
        self.COL_SOLVER   = 3
        self.COL_MODEL    = 4
        self.COL_SIZE     = 5
        self.COL_WEIGHT   = 6
        self.COL_COLOR    = 7
        #dirの位置をtreWidgetに渡す。
        self.treeWidget.COL_DIR = self.COL_DIR


    #
    #  createItemDict
    #-----------------
    def createItemDict(self, folders):
        """ dirをkeyとして、treeNodesに保存する内容の辞書を作成する。
        辞書の内容は、itemDefinesに従った内容。"""
        #itemの辞書作成
        itemDict = {}
        #rootを作成
        rootDir = folders[0]
        itemCont = [self.iconFolder, rootDir, "", "", "", "", self.NORMAL, "black"]
        itemDict[rootDir] = itemCont
        #rootDir以下を作成
        for folder in folders[1:]:
            #(folType, model, solType, solver, meshSize) = self.treeWidget.dataDict[folder]
            folderCont = self.treeWidget.dataDict[folder]
            folType = folderCont[0]
            icon = self.iconFolder
            if folType == "case":
                icon = self.iconCase
            itemName = os.path.basename(folder)
            itemCont = [icon, itemName] + folderCont[1:] + [self.NORMAL, "black"]
            itemDict[folder] = itemCont
        return itemDict

    #
    #  getFolderCont
    def getFolderCont(self, folderDir):
        """ folderの内容を取得する"""

        def isCaseDir(folderDir):
            if os.path.exists(folderDir + os.sep + "FistrModel.cnt") == True:
                return True
            return False

        def getModelName(folderDir):
            model = ""
            fileName = folderDir + os.sep + "easyIstrSub_data"
            if os.path.exists(fileName) == False:
                return model
            f = open(fileName, encoding="utf-8"); lines = f.readlines(); f.close()
            for line in lines:
                if line[:len("sourceMesh")] == "sourceMesh":
                    words = line.split()
                    if len(words) > 1:
                        model = words[1]
                    break
            return model

        def getCaseCont(folderDir):
            solType = ""
            solver = ""
            meshSize = ""
            fileName = folderDir + os.sep + "FistrModel.cnt"
            f = open(fileName, encoding="utf-8"); lines = f.readlines(); f.close()
            #solutionをチェック
            ist = 0
            for i in range(len(lines)):
                line = lines[i]
                if line[:len("!SOLUTION")].upper() == "!SOLUTION":
                    solType = abaqus2fistr.getValName(line, "TYPE").upper()
                    ist = i
                    break
            if solType == "HEAT":
                #HEATの場合は、定常・非定常を確認
                for i in range(ist, len(lines)):
                    line = lines[i]
                    if line[:len("!HEAT")].upper() == "!HEAT":
                        words = lines[i+1].replace(" ", "").split(",")
                        try:
                            a = float(words[0])
                            if a != 0.0:
                                #非定常
                                solType = "HEAT_dy"
                        except:
                            pass
            #solverを取得
            for line in lines:
                if line[:len("!SOLVER")].upper() == "!SOLVER":
                    solver = abaqus2fistr.getValName(line, "METHOD")
                    break
            #meshSizeを取得
            meshFile = folderDir + os.sep + "FistrModel.msh"
            if os.path.exists(meshFile):
                meshSize = "%10.3f" % (os.path.getsize(meshFile)/1000000)
                #meshSize = meshSize.replace(" ", "")
            return (solType, solver, meshSize)

        folType, model, solType, solver, meshSize = "folder", "", "", "", ""
        if isCaseDir(folderDir) == True:
            folType = "case"
            model = getModelName(folderDir)
            (solType, solver, meshSize) = getCaseCont(folderDir)
        return [folType, solType, solver, model, meshSize]

    #
    #  setSelectedDir
    #-----------------------
    def setSelectedDir(self):
        """ currDirをチェック付アイコン、bold、redに設定する"""
        if self.treeWidget.itemDict[self.currDir][self.COL_ICON] == self.iconFolder:
            icon = self.iconFolderCk
        elif self.treeWidget.itemDict[self.currDir][self.COL_ICON] == self.iconCase:
            icon = self.iconCaseCk
        self.treeWidget.changeColFromDir(self.currDir, self.COL_ICON, icon)
        self.treeWidget.changeColFromDir(self.currDir, self.COL_WEIGHT, self.BOLD)
        self.treeWidget.changeColFromDir(self.currDir, self.COL_COLOR, "red")

    #
    #  expandDir
    #--------------
    def expandDir(self, path):
        """ treeをexpandした時の処理。"""
        #pathのchildrenを全て削除（2ヶの辞書も削除）
        self.treeWidget.deleteChildrenPath(path)
        #path以下のfolderを取得
        expDirList = self.treeWidget.getDirFromTreePath(path)
        expDir = os.sep.join(expDirList)
        folders = self.treeWidget.getFolders(expDir)
        folders += self.treeWidget.getChildFolders(folders)
        folders.sort()
        #辞書に新しい内容を追加
        self.addNewDataToDict(folders)
        #treeDataを再作成
        itemDict = self.treeWidget.itemDict
        treeData = self.treeWidget.createTreeData(itemDict)
        self.treeWidget.treeData = treeData
        #parentDir以下を再作成
        items = self.treeWidget.getItemsFromTreeData(expDir)
        treeStore = self.treeWidget.treeStore
        parentIter = self.treeWidget.getIterInTreeView(expDir)
        self.treeWidget.addTreeNodes(treeStore, parentIter, items)
        #currDirを再設定（icon、書体、fontColor）
        self.setSelectedDir()
        #expDirを展開する
        self.maskEvent = True
        self.treeWidget.expandChildDir(expDir)
        self.maskEvent = False

    #
    #  addNewDataToDict
    def addNewDataToDict(self, folders):
        """ 辞書にnewDataを追加する"""
        for folder in folders:
            #dataDictに追加
            folderCont = self.getFolderCont(folder)
            self.treeWidget.dataDict[folder] = folderCont
            #itemDictに追加
            #(folType, solType, solver, meshSize) = folderCont
            folType = folderCont[0]
            #  iconを設定
            icon = self.iconFolder
            if folType == "case":
                icon = self.iconCase
            #  dir名を設定
            name = os.path.basename(folder)
            if folder == self.rootDir:
                name = folder
            #  itemContを作成
            itemCont = [icon, name] + folderCont[1:] + [self.NORMAL, "black"]
            self.treeWidget.itemDict[folder] = itemCont
    